@@ -14,7 +14,7 @@ use URI::Escape;
 
 use vars qw( $VERSION );
 
-$VERSION = '0.65';
+$VERSION = '0.81';
 
 =head1 NAME
 
@@ -156,6 +156,13 @@ sub differ {
                           return_output => 1,
                       );
 
+  # Return output as a string with HTTP headers omitted (for tests).
+  $guide->display_node(
+                          id            => "Calthorpe Arms",
+                          return_output => 1,
+                          noheaders     => 1,
+                      );
+
   # Or return the hash of variables that will be passed to the template
   # (not including those set additionally by OpenGuides::Template).
   $guide->display_node(
@@ -176,10 +183,13 @@ C<return_output> parameter is omitted.
 (At the moment, C<return_tt_vars> acts as if the C<intercept_redirect>
 parameter was passed.)
 
+The C<noheaders> parameter only takes effect if C<return_output> is true
+and C<intercept_redirect> is false or omitted.
+
 If you have specified the C<host_checker_module> option in your
 C<wiki.conf>, this method will attempt to call the <blacklisted_host>
 method of that module to determine whether the host requesting the node
-has been blacklisted. If this method returns true, then the 
+has been blacklisted. If this method returns true, then the
 C<blacklisted_host.tt> template will be used to display an error message.
 
 The C<blacklisted_host> method will be passed a scalar containing the host's
@@ -191,6 +201,8 @@ sub display_node {
     my ($self, %args) = @_;
     my $return_output = $args{return_output} || 0;
     my $intercept_redirect = $args{intercept_redirect};
+    my $noheaders = ( $return_output && !$intercept_redirect
+                      && $args{noheaders} );
     my $version = $args{version};
     my $id = $args{id} || $self->config->home_name;
     my $wiki = $self->wiki;
@@ -212,12 +224,13 @@ sub display_node {
 
     if ( $is_blacklisted ) {
         my $output = OpenGuides::Template->output(
-            wiki     => $self->wiki,
-            config   => $config,
-            template => "blacklisted_host.tt",
-            vars     => {
-                          not_editable => 1,
-                        },
+            wiki      => $self->wiki,
+            config    => $config,
+            template  => "blacklisted_host.tt",
+            vars      => {
+                           not_editable => 1,
+                         },
+            noheaders => $noheaders,
         );
         return $output if $return_output;
         print $output;
@@ -225,7 +238,7 @@ sub display_node {
     }
 
     $tt_vars{home_name} = $self->config->home_name;
-    
+
     if ( $id =~ /^(Category|Locale) (.*)$/ ) {
         my $type = $1;
         $tt_vars{is_indexable_node} = 1;
@@ -262,11 +275,11 @@ sub display_node {
                                         latitude => $metadata{latitude}[0],
                                         config => $config);
     if ($args{format} && $args{format} eq 'raw') {
-        print "Content-Type: text/plain\n\n";
+        print "Content-Type: text/plain\n\n" unless $noheaders;
         print $node_data{content};
         return 0;
     }
-   
+
     my %metadata_vars = OpenGuides::Template->extract_metadata_vars(
                             wiki     => $wiki,
                             config   => $config,
@@ -364,20 +377,22 @@ sub display_node {
         }
         return %tt_vars if $args{return_tt_vars};
         my $output = $self->process_template(
-                                                id            => $id,
-                                                template      => "home_node.tt",
-                                                tt_vars       => \%tt_vars,
-                                                http_status   => $http_status
+                                              id          => $id,
+                                              template    => "home_node.tt",
+                                              tt_vars     => \%tt_vars,
+                                              http_status => $http_status,
+                                              noheaders   => $noheaders,
                                             );
         return $output if $return_output;
         print $output;
     } else {
         return %tt_vars if $args{return_tt_vars};
         my $output = $self->process_template(
-                                                id            => $id,
-                                                template      => "node.tt",
-                                                tt_vars       => \%tt_vars,
-                                                http_status   => $http_status
+                                              id          => $id,
+                                              template    => "node.tt",
+                                              tt_vars     => \%tt_vars,
+                                              http_status => $http_status,
+                                              noheaders   => $noheaders,
                                             );
         return $output if $return_output;
         print $output;
@@ -604,7 +619,7 @@ sub preview_edit {
                                                set_coord_field_vars => 1,
     );
     foreach my $var ( qw( username comment edit_type ) ) {
-        $new_metadata{$var} = $q->escapeHTML($q->param($var));
+        $new_metadata{$var} = $q->escapeHTML(scalar $q->param($var));
     }
 
     if ($wiki->verify_checksum($node, $checksum)) {
@@ -637,7 +652,49 @@ sub preview_edit {
     }
 }
 
-=item B<display_recent_changes>  
+=item B<display_prefs_form>
+
+  $guide->display_prefs_form;
+
+Displays a form that lets the user view and set their preferences.  The
+C<return_output> and C<return_tt_vars> parameters can be used to return
+the output or template variables, instead of printing the output to STDOUT.
+The C<noheaders> parameter can also be used in conjunction with
+C<return_output>, if you wish to omit all HTTP headers.
+
+=cut
+
+sub display_prefs_form {
+    my ($self, %args) = @_;
+    my $config = $self->config;
+    my $wiki = $self->wiki;
+
+    my $from = $ENV{HTTP_REFERER} || "";
+    my $url_base = $config->script_url . $config->script_name;
+    if ( $from !~ /^$url_base/ ) {
+        $from = "";
+    }
+
+    my %tt_vars = (
+                    not_editable  => 1,
+                    show_form     => 1,
+                    not_deletable => 1,
+                    return_to_url => $from,
+    );
+    return %tt_vars if $args{return_tt_vars};
+
+    my $output = OpenGuides::Template->output(
+        wiki      => $wiki,
+        config    => $config,
+        template  => "preferences.tt",
+	vars      => \%tt_vars,
+        noheaders => $args{noheaders},
+    );
+    return $output if $args{return_output};
+    print $output;
+}
+
+=item B<display_recent_changes>
 
   $guide->display_recent_changes;
 
@@ -654,68 +711,41 @@ sub display_recent_changes {
     my $id = $args{id} || $self->config->home_name;
     my $return_output = $args{return_output} || 0;
     my (%tt_vars, %recent_changes);
+    # NB the $q stuff below should be removed - we should _always_ do this via
+    # an argument to the method.
     my $q = CGI->new;
-    my $since = $q->param("since");
+    my $since = $args{since} || $q->param("since");
     if ( $since ) {
         $tt_vars{since} = $since;
         my $t = localtime($since); # overloaded by Time::Piece
         $tt_vars{since_string} = $t->strftime;
-        my %criteria = ( since => $since );   
+        my %criteria = ( since => $since );
         $criteria{metadata_was} = { edit_type => "Normal edit" }
           unless $minor_edits;
-        my @rc = $self->{wiki}->list_recent_changes( %criteria );
-
-        my $base_url = $config->script_name . '?';
-        
-        @rc = map {
-            {
-              name        => CGI->escapeHTML($_->{name}),
-              last_modified => CGI->escapeHTML($_->{last_modified}),
-              version     => CGI->escapeHTML($_->{version}),
-              comment     => OpenGuides::Utils::parse_change_comment(
-                  CGI->escapeHTML($_->{metadata}{comment}[0]),
-                  $base_url,
-              ),
-              username    => CGI->escapeHTML($_->{metadata}{username}[0]),
-              host        => CGI->escapeHTML($_->{metadata}{host}[0]),
-              username_param => CGI->escape($_->{metadata}{username}[0]),
-              edit_type   => CGI->escapeHTML($_->{metadata}{edit_type}[0]),
-              url         => $base_url
-      . CGI->escape($wiki->formatter->node_name_to_node_param($_->{name})),
-        }
-                   } @rc;
+        my @rc = $self->_get_recent_changes(
+                     config => $config, criteria => \%criteria );
         if ( scalar @rc ) {
-            $recent_changes{since} = \@rc; 
+            $recent_changes{since} = \@rc;
         }
     } else {
+        # Look at day, week, fortnight, month separately, but make sure things
+        # don't appear in e.g. "week" if we've already seen them in "day".
+        my %seen;
         for my $days ( [0, 1], [1, 7], [7, 14], [14, 30] ) {
             my %criteria = ( between_days => $days );
             $criteria{metadata_was} = { edit_type => "Normal edit" }
               unless $minor_edits;
-            my @rc = $self->{wiki}->list_recent_changes( %criteria );
-
-            my $base_url = $config->script_name . '?';
-            
-            @rc = map {
-            {
-              name        => CGI->escapeHTML($_->{name}),
-              last_modified => CGI->escapeHTML($_->{last_modified}),
-              version     => CGI->escapeHTML($_->{version}),
-              comment     => OpenGuides::Utils::parse_change_comment(
-                  CGI->escapeHTML($_->{metadata}{comment}[0]),
-                  $base_url,
-              ),
-              username    => CGI->escapeHTML($_->{metadata}{username}[0]),
-              host        => CGI->escapeHTML($_->{metadata}{host}[0]),
-              username_param => CGI->escape($_->{metadata}{username}[0]),
-              edit_type   => CGI->escapeHTML($_->{metadata}{edit_type}[0]),
-              url         => $base_url
-      . CGI->escape($wiki->formatter->node_name_to_node_param($_->{name})),
-        }
-                       } @rc;
-            if ( scalar @rc ) {
-                $recent_changes{$days->[1]} = \@rc;
-        }
+            my @rc = $self->_get_recent_changes(
+                         config => $config, criteria => \%criteria );
+            my @filtered;
+            foreach my $node ( @rc ) {
+                next if $seen{$node->{name}};
+                $seen{$node->{name}}++;
+                push @filtered, $node;
+            }
+            if ( scalar @filtered ) {
+                $recent_changes{$days->[1]} = \@filtered;
+            }
         }
     }
     $tt_vars{not_editable} = 1;
@@ -737,6 +767,52 @@ sub display_recent_changes {
     print $output;
 }
 
+sub _get_recent_changes {
+    my ( $self, %args ) = @_;
+    my $wiki = $self->wiki;
+    my $formatter = $wiki->formatter;
+    my $config = $self->config;
+    my %criteria = %{ $args{criteria} };
+
+    my @rc = $wiki->list_recent_changes( %criteria );
+    my $base_url = $config->script_name . '?';
+
+    # If using metadata_was then we need to pick out just the most recent
+    # versions.
+    if ( $criteria{metadata_was} ) {
+        my %seen;
+        my @filtered;
+        foreach my $node ( @rc ) {
+            next if $seen{$node->{name}};
+            $seen{$node->{name}}++;
+            push @filtered, $node;
+        }
+        @rc = @filtered;
+    }
+
+    @rc = map {
+        my $url = $base_url
+                . CGI->escape($formatter->node_name_to_node_param($_->{name}));
+        # CGI->escape escapes commas in URLs.  This is annoying.
+        $url =~ s/%2C/,/gs;
+        {
+          name => CGI->escapeHTML($_->{name}),
+          last_modified => CGI->escapeHTML($_->{last_modified}),
+          version => CGI->escapeHTML($_->{version}),
+          comment => OpenGuides::Utils::parse_change_comment(
+                         CGI->escapeHTML($_->{metadata}{comment}[0]),
+                         $base_url,
+                     ),
+          username => CGI->escapeHTML($_->{metadata}{username}[0]),
+          host => CGI->escapeHTML($_->{metadata}{host}[0]),
+          username_param => CGI->escape($_->{metadata}{username}[0]),
+          edit_type => CGI->escapeHTML($_->{metadata}{edit_type}[0]),
+          url => $url
+        }
+    } @rc;
+    return @rc;
+}
+
 =item B<display_diffs>
 
   $guide->display_diffs(
@@ -846,34 +922,66 @@ sub show_backlinks {
 
 =item B<show_index>
 
+  # Show everything in Category: Pubs.
   $guide->show_index(
-                        type   => "category",
-                        value  => "pubs",
+                        cat => "pubs",
                     );
 
-  # RDF version.
+  # Show all pubs in Holborn.
   $guide->show_index(
-                        type   => "locale",
-                        value  => "Holborn",
+                        cat => "pubs",
+                        loc => "holborn",
+                    );
+
+  # RDF version of things in Locale: Holborn.
+  $guide->show_index(
+                        loc  => "Holborn",
                         format => "rdf",
                     );
 
   # RSS / Atom version (recent changes style).
   $guide->show_index(
-                        type   => "locale",
-                        value  => "Holborn",
+                        loc    => "Holborn",
                         format => "rss",
                     );
 
   # Or return output as a string (useful for writing tests).
   $guide->show_index(
-                        type          => "category",
-                        value         => "pubs",
+                        cat           => "pubs",
                         return_output => 1,
                     );
 
-If either the C<type> or the C<value> parameter is omitted, then all pages
-will be returned.
+  # Return output as a string with HTTP headers omitted (for tests).
+  $guide->show_index(
+                        cat           => "pubs",
+                        return_output => 1,
+                        noheaders     => 1,
+                    );
+
+  # Or return the template variables (again, useful for writing tests).
+  $guide->show_index(
+                        cat            => "pubs",
+                        format         => "map"
+                        return_tt_vars => 1,
+                    );
+
+If neither C<cat> or C<loc> is supplied, then all pages will be returned.
+
+The recommended format of parameters to this method changed to the
+above in version 0.67 of OpenGuides, though older invocations are
+still supported and will redirect to the new URL format.
+
+If you pass the C<return_output> or C<return_tt_vars> parameters, and a
+redirect is required, this method will fake the redirect and return the
+output/variables that will actually end up being viewed by the user.  If
+instead you want to see the HTTP headers that will be printed in order to
+perform the redirect, pass the C<intercept_redirect> parameter as well.
+
+The C<intercept_redirect> parameter has no effect if no redirect is required,
+or if the C<return_output>/C<return_tt_vars> parameter is omitted.
+
+The C<noheaders> parameter only takes effect if C<return_output> is true
+and C<intercept_redirect> is false or omitted.
 
 =cut
 
@@ -881,6 +989,7 @@ sub show_index {
     my ($self, %args) = @_;
     my $wiki = $self->wiki;
     my $formatter = $wiki->formatter;
+    my $use_leaflet = $self->config->use_leaflet;
     my %tt_vars;
     my @selnodes;
 
@@ -895,29 +1004,86 @@ sub show_index {
             };
             $tt_vars{not_editable} = 1;
         } else {
-            @selnodes = $wiki->list_nodes_by_metadata(
-                metadata_type  => $args{type},
-                metadata_value => $args{value},
-                ignore_case    => 1
-            );
-            my $name = ucfirst($args{type}) . " $args{value}";
-            my $url = $self->config->script_name
-                      . "?"
-                      . ucfirst( $args{type} )
-                      . "_"
-                      . uri_escape(
-                                      $formatter->node_name_to_node_param($args{value})
-                                  );
-            $tt_vars{criterion} = {
-                type  => $args{type},
-                value => $args{value}, # for RDF version
-                name  => CGI->escapeHTML( $name ),
-                url   => $url
-            };
-            $tt_vars{not_editable} = 1;
+            return $self->_do_old_style_index_search( %args );
         }
     } else {
-        @selnodes = $wiki->list_all_nodes();
+        # OK, we either show everything, or do a new-style cat/loc search.
+        my $cat = $args{cat} || "";
+        my $loc = $args{loc} || "";
+        my ( $type, $value, @names, @criteria );
+        if ( !$cat && !$loc ) {
+            @selnodes = $wiki->list_all_nodes();
+        } else {
+            my ( @catnodes, @locnodes );
+            if ( $cat ) {
+                @catnodes = $wiki->list_nodes_by_metadata(
+                    metadata_type  => "category",
+                    metadata_value => $cat,
+                    ignore_case    => 1
+                );
+                my $name = "Category $cat";
+                $name =~ s/(\s\w)/\U$1/g;
+                push @criteria, {
+                    type  => "category",
+                    value => $cat,
+                    name  => $name,
+                    param => $formatter->node_name_to_node_param( $name ),
+                };
+                push @names, $name;
+            }
+            if ( $loc ) {
+                @locnodes = $wiki->list_nodes_by_metadata(
+                    metadata_type  => "locale",
+                    metadata_value => $loc,
+                    ignore_case    => 1
+                );
+                my $name = "Locale $loc";
+                $name =~ s/(\s\w)/\U$1/g;
+                push @criteria, {
+                    type  => "locale",
+                    value => $loc,
+                    name  => $name,
+                    param => $formatter->node_name_to_node_param( $name ),
+                };
+                push @names, $name;
+            }
+            if ( $cat && !$loc ) {
+                @selnodes = @catnodes;
+            } elsif ( $loc && !$cat ) {
+                @selnodes = @locnodes;
+            } else {
+                # Intersect the category and locale results.
+                my %count = ();
+                foreach my $node ( @catnodes, @locnodes ) { $count{$node}++; }
+                foreach my $node ( keys %count ) {
+                    push @selnodes, $node if $count{$node} > 1;
+                }
+            }
+            $tt_vars{criteria_title} = join( " and ", @names );
+            $tt_vars{criteria} = \@criteria;
+            $tt_vars{not_editable} = 1;
+        }
+
+        $tt_vars{page_description} =
+            OpenGuides::Utils->get_index_page_description(
+                format => $args{format} || "",
+                criteria => \@criteria,
+            );
+
+        my $feed_base = $self->config->script_url
+                        . $self->config->script_name . "?action=index";
+        foreach my $criterion ( @criteria ) {
+            if ( $criterion->{type} eq "category" ) {
+              $feed_base .= ";cat=" . lc( $criterion->{value} );
+            } elsif ( $criterion->{type} eq "locale" ) {
+              $feed_base .= ";loc=" . lc( $criterion->{value} );
+            }
+        }
+        my @dropdowns = OpenGuides::CGI->make_index_form_dropdowns(
+                guide => $self,
+                selected => \@criteria );
+        $tt_vars{index_form_fields} = \@dropdowns;
+        $tt_vars{feed_base} = $feed_base;
     }
 
     my @nodes = map {
@@ -927,7 +1093,9 @@ sub show_index {
                             param     => $formatter->node_name_to_node_param($_) }
                         } sort @selnodes;
 
-    # Convert the lat+long to WGS84 as required
+    # Convert the lat+long to WGS84 as required, and count how many nodes
+    # we have for the map (if using Leaflet).
+    my $nodes_on_map;
     for(my $i=0; $i<scalar @nodes;$i++) {
         my $node = $nodes[$i];
         if($node) {
@@ -943,6 +1111,15 @@ sub show_index {
 
             push @{$nodes[$i]->{node_data}->{metadata}->{wgs84_long}}, $wgs84_long;
             push @{$nodes[$i]->{node_data}->{metadata}->{wgs84_lat}},  $wgs84_lat;
+            if ( $use_leaflet ) {
+                if ( defined $wgs84_lat && $wgs84_lat =~ /^[-.\d]+$/
+                     && defined $wgs84_long && $wgs84_long =~ /^[-.\d]+$/ ) {
+                    $node->{has_geodata} = 1;
+                    $node->{wgs84_lat} = $wgs84_lat;
+                    $node->{wgs84_long} = $wgs84_long;
+                    $nodes_on_map++;
+                }
+            }
         }
     }
 
@@ -961,26 +1138,49 @@ sub show_index {
             $template = "plain_index.tt";
             $conf{content_type} = "text/plain";
         } elsif ( $args{format} eq "map" ) {
-            my $q = CGI->new;
-            $tt_vars{zoom} = $q->param('zoom') || '';
-            $tt_vars{lat} = $q->param('lat') || '';
-            $tt_vars{long} = $q->param('long') || '';
-            $tt_vars{map_type} = $q->param('map_type') || '';
-            $tt_vars{centre_long} = $self->config->centre_long;
-            $tt_vars{centre_lat} = $self->config->centre_lat;
-            $tt_vars{default_gmaps_zoom} = $self->config->default_gmaps_zoom;
-            $tt_vars{enable_gmaps} = 1;
             $tt_vars{display_google_maps} = 1; # override for this page
-            $template = "map_index.tt";
-            
+            if ( $use_leaflet ) {
+                if ( $nodes_on_map ) {
+                    my @points = map {
+                    {  wgs84_lat =>
+                           $_->{node_data}->{metadata}->{wgs84_lat}[0],
+                      wgs84_long =>
+                           $_->{node_data}->{metadata}->{wgs84_long}[0]
+                    }
+                                     } @nodes;
+                    my %minmaxdata = OpenGuides::Utils->get_wgs84_min_max(
+                        nodes => \@points );
+                    %tt_vars = ( %tt_vars, %minmaxdata );
+                } else {
+                    $tt_vars{no_nodes_on_map} = 1;
+                }
+                $template = "map_index_leaflet.tt";
+            } else {
+                my $q = CGI->new;
+                $tt_vars{zoom} = $q->param('zoom') || '';
+                $tt_vars{lat} = $q->param('lat') || '';
+                $tt_vars{long} = $q->param('long') || '';
+                $tt_vars{map_type} = $q->param('map_type') || '';
+                $tt_vars{centre_long} = $self->config->centre_long;
+                $tt_vars{centre_lat} = $self->config->centre_lat;
+                $tt_vars{default_gmaps_zoom}
+                                      = $self->config->default_gmaps_zoom;
+                $tt_vars{enable_gmaps} = 1;
+                $template = "map_index.tt";
+            }
         } elsif( $args{format} eq "rss" || $args{format} eq "atom") {
             # They really wanted a recent changes style rss/atom feed
             my $feed_type = $args{format};
             my ($feed,$content_type) = $self->get_feed_and_content_type($feed_type);
-            $feed->set_feed_name_and_url_params(
-                        "Index of $args{type} $args{value}",
-                        "action=index;index_type=$args{type};index_value=$args{value}"
-            );
+            my ($name, $params );
+            if ( $args{cat} ) {
+                $name = "Index of Category $args{cat}";
+                $params = "action=index;cat=$args{cat}";
+            } else {
+                $name = "Index of Locale $args{loc}";
+                $params = "action=index;loc=$args{loc}";
+            }
+            $feed->set_feed_name_and_url_params( $name, $params );
 
             # Grab the actual node data out of @nodes
             my @node_data;
@@ -1000,17 +1200,45 @@ sub show_index {
         $template = "site_index.tt";
     }
 
+    return %tt_vars if $args{return_tt_vars};
+
     %conf = (
                 %conf,
                 template    => $template,
                 tt_vars     => \%tt_vars,
             );
 
+    if ( $args{return_output} && !$args{intercept_redirect} ) {
+        $conf{noheaders} = $args{noheaders};
+    }
+
     my $output = $self->process_template( %conf );
     return $output if $args{return_output};
     print $output;
 }
 
+# Deal with legacy URLs/tests.
+sub _do_old_style_index_search {
+    my ( $self, %args ) = @_;
+    if ( ( $args{return_output} || $args{return_tt_vars} ) ) {
+        if ( $args{intercept_redirect} ) {
+            return $self->redirect_index_search( %args );
+        } else {
+            my $type = delete $args{type};
+            my $value = delete $args{value};
+            if ( $type eq "category" ) {
+                return $self->show_index( %args, cat => $value );
+            } elsif ( $type eq "locale" ) {
+                return $self->show_index( %args, loc => $value );
+            } else {
+                return $self->show_index( %args );
+            }
+        }
+    } else {
+        print $self->redirect_index_search( %args );
+    }
+}
+
 =item B<show_metadata>
 
   $guide->show_metadata();
@@ -1057,7 +1285,7 @@ sub show_metadata {
     if($args{"format"}) {
        if($args{"format"} eq "json") {
           $content_type = "text/javascript";
-          my $json = OpenGuides::JSON->new( wiki => $wiki, 
+          my $json = OpenGuides::JSON->new( wiki => $wiki,
                                             config => $self->config );
           $output = $json->output_as_json(
                                  $type => \@values
@@ -1182,10 +1410,10 @@ sub get_feed_and_content_type {
                          category     => "Pubs",
                      );
 
-C<feed_type> is a mandatory parameter. Supported values at present are 
+C<feed_type> is a mandatory parameter. Supported values at present are
 "rss" and "atom".
 
-C<feed_listing> is a mandatory parameter. Supported values at present 
+C<feed_listing> is a mandatory parameter. Supported values at present
 are "recent_changes". (More values are coming soon though!)
 
 As with other methods, the C<return_output> parameter can be used to
@@ -1201,7 +1429,7 @@ sub display_feed {
 
     my $feed_listing = $args{feed_listing};
     croak "No feed listing given" unless $feed_listing;
-    
+
     my $return_output = $args{return_output} ? 1 : 0;
 
     # Basic criteria, whatever the feed listing type is
@@ -1242,12 +1470,12 @@ sub display_feed {
         $output .= "; charset=".$self->config->http_charset;
     }
     $output .= "\n";
-    
+
     # Get the feed, and the timestamp, in one go
-    my ($feed_output, $feed_timestamp) = 
+    my ($feed_output, $feed_timestamp) =
         $feed->make_feed( %criteria );
     my $maker = $feed->fetch_maker($feed_type);
- 
+
     $output .= "Last-Modified: " . ($maker->parse_feed_timestamp($feed_timestamp))->strftime('%a, %d %b %Y %H:%M:%S +0000') . "\n\n";
     $output .= $feed_output;
 
@@ -1279,7 +1507,7 @@ sub display_about {
   <name>OpenGuides</name>
 
   <created>2003-04-29</created>
-  
+
   <shortdesc xml:lang="en">
     A wiki engine for collaborative description of places with specialised
     geodata metadata features.
@@ -1318,10 +1546,10 @@ sub display_about {
   </release>
 
   <download-page rdf:resource="http://search.cpan.org/dist/OpenGuides/" />
-  
+
   <!-- Freshmeat category: Internet :: WWW/HTTP :: Dynamic Content -->
   <category rdf:resource="http://freshmeat.net/browse/92/" />
-  
+
   <license rdf:resource="http://www.opensource.org/licenses/gpl-license.php" />
   <license rdf:resource="http://www.opensource.org/licenses/artistic-license.php" />
 
@@ -1419,7 +1647,7 @@ information</a>
 </body>
 </html>};
     }
-    
+
     return $output if $args{return_output};
     print $output;
 }
@@ -1575,7 +1803,7 @@ sub commit_node {
                                           metadata => \%new_metadata
         );
     }
-    
+
     my $written = $wiki->write_node( $node, $content, $checksum,
                                      \%new_metadata );
 
@@ -1743,6 +1971,10 @@ sub _autoCreateCategoryLocale {
                               );
                 my $ok = $tt->process( "custom_autocreate_content.tt",
                                        \%tt_vars, \$blurb );
+                if ( !$ok ) {
+                    $ok = $tt->process( "autocreate_content.tt",
+                                        \%tt_vars, \$blurb );
+                }
                 if ( !$ok ) {
                     $blurb = "\@INDEX_LINK [[$node]]";
                 }
@@ -2005,8 +2237,8 @@ sub moderate_node {
 
 =item B<show_missing_metadata>
 
-Search for nodes which don't have a certain kind of metadata. Optionally
-also excludes Locales and Categories
+Search for nodes which don't have a certain kind of metadata.  Excludes nodes
+which are pure redirects, and optionally also excludes locales and categories.
 
 =cut
 
@@ -2018,65 +2250,89 @@ sub show_missing_metadata {
     my $wiki = $self->wiki;
     my $formatter = $self->wiki->formatter;
     my $script_name = $self->config->script_name;
+    my $use_leaflet = $self->config->use_leaflet;
 
-    my ($metadata_type, $metadata_value, $exclude_locales, $exclude_categories)
-        = @args{ qw( metadata_type metadata_value exclude_locales exclude_categories ) };
+    my ( $metadata_type, $metadata_value, $exclude_locales,
+         $exclude_categories, $format)
+        = @args{ qw( metadata_type metadata_value exclude_locales
+                     exclude_categories format ) };
+    $format ||= "";
 
     my @nodes;
     my $done_search = 0;
+    my $nodes_on_map;
 
     # Only search if they supplied at least a metadata type
     if($metadata_type) {
         $done_search = 1;
-        @nodes = $wiki->list_nodes_by_missing_metadata(
+        my @all_nodes = $wiki->list_nodes_by_missing_metadata(
                             metadata_type => $metadata_type,
                             metadata_value => $metadata_value,
                             ignore_case    => 1,
         );
 
-        # Do we need to filter some nodes out?
-        if($exclude_locales || $exclude_categories) {
-            my @all_nodes = @nodes;
-            @nodes = ();
-
-            foreach my $node (@all_nodes) {
-                if($exclude_locales && $node =~ /^Locale /) { next; }
-                if($exclude_categories && $node =~ /^Category /) { next; }
-                push @nodes, $node;
+        # Filter out redirects; also filter out locales/categories if required.
+        foreach my $node ( sort @all_nodes ) {
+            next if ( $exclude_locales && $node =~ /^Locale / );
+            next if ( $exclude_categories && $node =~ /^Category / );
+            my %data = $wiki->retrieve_node( $node );
+            next if OpenGuides::Utils->detect_redirect(
+                        content => $data{content} );
+            my $node_param = $formatter->node_name_to_node_param( $node );
+            my %this_node = (
+                name => $node,
+                param => $node_param,
+                address => $data{metadata}{address}[0],
+                view_url => "$script_name?$node_param",
+                edit_url => "$script_name?id=$node_param;action=edit",
+            );
+            if ( $format eq "map" && $use_leaflet ) {
+                my ( $wgs84_long, $wgs84_lat )
+                    = OpenGuides::Utils->get_wgs84_coords(
+                          latitude  => $data{metadata}{latitude}[0],
+                          longitude => $data{metadata}{longitude}[0],
+                          config    => $self->config );
+                if ( defined $wgs84_lat ) {
+                    $this_node{has_geodata} = 1;
+                    $this_node{wgs84_lat} = $wgs84_lat;
+                    $this_node{wgs84_long} = $wgs84_long;
+                    $nodes_on_map++;
+                }
             }
+            push @nodes, \%this_node;
         }
     }
 
-    # Build nice edit etc links for our nodes
-    my @tt_nodes;
-    for my $node (@nodes) {
-        my %n;
-
-        # Make the URLs
-        my $node_param = uri_escape( $formatter->node_name_to_node_param( $node ) );
-
-        # Save into the hash
-        $n{'name'} = $node;
-        $n{'view_url'} = $script_name . "?id=" . $node_param;
-        $n{'edit_url'} = $script_name . "?id=" . $node_param . ";action=edit";
-        push @tt_nodes, \%n;
-    }
-
     # Set up our TT variables, including the search parameters
     my %tt_vars = (
                       not_editable  => 1,
                       not_deletable => 1,
                       deter_robots  => 1,
-
-                      nodes => \@tt_nodes,
+                      nodes         => \@nodes,
                       done_search    => $done_search,
+                      no_nodes_on_map => !$nodes_on_map,
                       metadata_type  => $metadata_type,
                       metadata_value => $metadata_value,
                       exclude_locales => $exclude_locales,
                       exclude_categories => $exclude_categories,
-
                       script_name => $script_name
                   );
+
+    # Figure out the map boundaries and centre, if applicable.
+    if ( $format eq "map" ) {
+        if ( $use_leaflet ) {
+            my %minmaxdata = OpenGuides::Utils->get_wgs84_min_max(
+                nodes => \@nodes );
+            if ( scalar %minmaxdata ) {
+                %tt_vars = ( %tt_vars, %minmaxdata );
+            }
+            $tt_vars{display_google_maps} = 1; # to get the JavaScript in
+        }
+        # Set the show_map var even if we don't have Leaflet enabled, so
+        # people aren't left wondering why there's no map.
+        $tt_vars{show_map} = 1;
+    }
+
     return %tt_vars if $return_tt_vars;
 
     # Render to the page
@@ -2084,6 +2340,7 @@ sub show_missing_metadata {
                                            id       => "",
                                            template => "missing_metadata.tt",
                                            tt_vars  => \%tt_vars,
+                                           noheaders => $args{noheaders} || 0,
                                         );
     return $output if $return_output;
     print $output;
@@ -2174,7 +2431,7 @@ sub revert_user_interface {
 
 =item B<display_admin_interface>
 
-Fetch everything we need to display the admin interface, and passes it off 
+Fetch everything we need to display the admin interface, and passes it off
  to the template
 
 =cut
@@ -2266,7 +2523,8 @@ sub process_template {
                           template    => $args{template},
                           vars        => $args{tt_vars},
                           cookies     => $args{cookies},
-                          http_status => $args{http_status}
+                          http_status => $args{http_status},
+                          noheaders   => $args{noheaders},
                       );
     if ( $args{content_type} ) {
         $output_conf{content_type} = $args{content_type};
@@ -2274,9 +2532,32 @@ sub process_template {
     return OpenGuides::Template->output( %output_conf );
 }
 
+# Redirection for legacy URLs.
+sub redirect_index_search {
+    my ( $self, %args ) = @_;
+    my $type   = lc( $args{type} || "" );
+    my $value  = lc( $args{value} || "" );
+    my $format = lc( $args{format} || "" );
+
+    my $script_url = $self->config->script_url;
+    my $script_name = $self->config->script_name;
+
+    my $url = "$script_url$script_name?action=index";
+
+    if ( $type eq "category" ) {
+        $url .= ";cat=$value";
+    } elsif ( $type eq "locale" ) {
+        $url .= ";loc=$value";
+    }
+    if ( $format ) {
+        $url .= ";format=$format";
+    }
+    return CGI->redirect( -uri => $url, -status => 301 );
+}
+
 sub redirect_to_node {
     my ($self, $node, $redirected_from) = @_;
-    
+
     my $script_url = $self->config->script_url;
     my $script_name = $self->config->script_name;
     my $formatter = $self->wiki->formatter;
@@ -2289,7 +2570,7 @@ sub redirect_to_node {
     $redir_param .= 'id=' if $oldid;
     $redir_param .= $id;
     $redir_param .= ";oldid=$oldid" if $oldid;
-    
+
     my $q = CGI->new;
     return $q->redirect( $redir_param );
 }
@@ -2309,15 +2590,15 @@ sub get_cookie {
 UTF8 data are currently not handled correctly throughout.
 
 Other bugs are documented at
-L<http://dev.openguides.org/>
+L<https://github.com/OpenGuides/OpenGuides/issues>
 
 =head1 SEE ALSO
 
 =over 4
 
-=item * The Open Guide to London, at L<http://london.openguides.org/>, is the first and biggest OpenGuides site.
+=item * The Randomness Guide to London, at L<http://london.randomness.org.uk/>, the largest OpenGuides site.
 
-=item * A list of live OpenGuides installs is available at L<http://openguides.org/>.
+=item * The list of live OpenGuides installs at L<http://openguides.org/>.
 
 =item * L<Wiki::Toolkit>, the Wiki toolkit which does the heavy lifting for OpenGuides.
 
@@ -2329,7 +2610,7 @@ If you have a question, a bug report, or a patch, or you're interested
 in joining the development team, please contact openguides-dev@lists.openguides.org
 (moderated mailing list, will reach all current developers but you'll have
 to wait for your post to be approved) or file a bug report at
-L<http://dev.openguides.org/>
+L<https://github.com/OpenGuides/OpenGuides/issues>
 
 =head1 AUTHOR
 
@@ -2337,7 +2618,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-     Copyright (C) 2003-2010 The OpenGuides Project.  All Rights Reserved.
+     Copyright (C) 2003-2013 The OpenGuides Project.  All Rights Reserved.
 
 The OpenGuides distribution is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
